"""
The main module of the product...contains commands that create an instance of the product...
contains what were previously external methods and their support functions before
Zope-productizing the software (now the external methods have been moved to the ZKnowMan class
as its methods and the supporting functions are at the ZKnowMan module level).
"""

#Start of imports related to Zope product infrastructure----------------------------------------------

from Globals import HTMLFile
from AccessControl.Permission import Permission
from Globals import MessageDialog
import AccessControl
import OFS
import StringIO

#End of imports related to Zope product infrastructure------------------------------------------------

from zkmCommand import ZkmCommand
from zkmHtml import formatError, formatMessage, RETRY_OR_SEE_LOG
from zkmRequest import ZkmRequest
from zkmSocket import ZkmSocket
from zkmTrace import writeTrace, writeError, writeTracebackAsString

import zkmFile
import zkmUrl

import os
import urllib
import string
import time

#Start of module-level Zope product infrastructure code-----------------------------------------------

def GetHTML(name):
    return HTMLFile(name, globals())

ZKnowMan_pages = {
    "zkmAddUser_html" :
    (GetHTML("zkmAddUser_html"), "Create a new ZKnowMan User"),
    "zkmSearch_html" :
    (GetHTML("zkmSearch_html"), "Search for cached pages containing query word(s)"),
    "zkmSurf_html" :
    (GetHTML("zkmSurf_html"), "Search and cache pages starting at specified URL")
}

def addDocsToFolder(theFolder, theDocs):
    """add the listed docs (theDocs is a dictionary) to the given folder"""
    errs = StringIO.StringIO()

    for key in theDocs.keys():
        id = key
        dtml, title = theDocs[key]

        try:
            thedtml = open(dtml.raw).read()
        except:
            raise RuntimeError, "Can't find " + dtml.raw

        try:
            theFolder.manage_addDocument(id, title=title, file=thedtml)
        except:
            traceback.print_exc(file=errs)

    errs.seek(0)
    return errs.read()

#shows the form for creating a new instance of the product ZKnowMan
addZKnowManForm = HTMLFile("addZKnowManForm_html", globals())

def addZKnowMan(self, id, title, tempFolder, mgFolder,
                htmlFilesFolder, binariesFolder, serverPort,
                adzapperConfigFileFolder, REQUEST=None):
    """Create a ZKnowMan product instance."""

    #create ZKnowMan product object and init with data from addZKnowManForm form
    handle = ZKnowMan()
    handle.id = id
    handle.title = title
    handle.tempFolder = tempFolder
    handle.mgFolder = mgFolder
    handle.htmlFilesFolder = htmlFilesFolder
    handle.binariesFolder = binariesFolder
    handle.serverPort = serverPort
    handle.adzapperConfigFileFolder = adzapperConfigFileFolder

    #create a user folder
    folder = AccessControl.User.UserFolder()
    handle._setObject("acl_users", folder)

    #create the role "ZKMUser"
    handle._addRole("ZKMUser")

    #create the product object in Zope
    self._setObject(id, handle)

    #set the permissions for the role "ZKMUser"
    p_list = ["View"]

    for permission in p_list:
        p = Permission(permission, "", handle)
        p.setRole("ZKMUser", permission in p_list)

    #create the DTML methods
    addDocsToFolder(handle, ZKnowMan_pages)

    #changing the View permission setting of the ZKnowMan instance
    old_permissions = self.permission_settings()
    new_permissions = []
    permissions_for_ZKMUsers = []

    for item in old_permissions:
        if item["name"] == "View":
            permissions_for_ZKMUsers.append(item["name"])
        else:
            new_permissions.append(item["name"])

    #---------must have Manager permission to add a new ZKM user
    handle.zkmAddUser_html.manage_acquiredPermissions(new_permissions)

    #---------must have ZKMUser role to surf and cache pages and binaries
    handle.zkmSurf_html.manage_acquiredPermissions(new_permissions)
    handle.zkmSurf_html.manage_role("ZKMUser", permissions_for_ZKMUsers)

    #---------must have ZKMUser role to search cached pages and binaries
    handle.zkmSearch_html.manage_acquiredPermissions(new_permissions)
    handle.zkmSearch_html.manage_role("ZKMUser", permissions_for_ZKMUsers)

    #create command files for zkmServer and zkmBatchDownloader
    createCommandFiles()

    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

#End of module-level Zope product infrastructure code-------------------------------------------------

#Start of ZKnowMan class support functions------------------------------------------------------------

def createTempFile(tempFolder, userName, htmlPageString):
    try:
        filePathName = zkmFile.createTempFileName(tempFolder, userName)
        f = open(filePathName, "w")
        f.write(htmlPageString)
        f.close()
    except:
        try:
            os.remove(filePathName)
        except:
            pass
        raise

    return filePathName

def createSavePageCommand(aZkmRequest, filePathName):
    userName = aZkmRequest.userName
    url = aZkmRequest.urlAfterOpen
    graphicsCaching = aZkmRequest.graphicsCaching

    dateAndTime = `time.time()`
    aZkmCommand = ZkmCommand(command="save", userName=userName, url=url,
                             filePathName=filePathName, graphicsCaching=graphicsCaching,
                             dateAndTime=dateAndTime)
    return aZkmCommand

def saveAndIndexPage(command, serverName, serverPort):
    aZkmSocket = ZkmSocket(command, serverName, serverPort)

    if not aZkmSocket.sendCommandNoResponse():
        raise Exception, "Error sending 'save' command."

    return

def createProxifyCommand(aZkmRequest, filePathName):
    url = aZkmRequest.urlAfterOpen
    graphicsCaching = aZkmRequest.graphicsCaching

    aZkmCommand = ZkmCommand(command="proxify", url=url, filePathName=filePathName,
                             graphicsCaching=graphicsCaching)
    return aZkmCommand

def proxifyPage(command, serverName, serverPort):
    try:
        aZkmSocket = ZkmSocket(command, serverName, serverPort)

        response = aZkmSocket.sendCommandGetResponse()

        if not response[0]:
            writeError("Error in 'proxify' command.", command=command)
        else:
            result = response[1]
            return result

    except Exception, data:
        writeError("Error retrieving page: " + `data`)
        writeTrackbackAsString(20)

    result = formatError("Error 'proxifying' page. " + RETRY_OR_SEE_LOG)

    return result

def createRetrieveCommand(userName,pageId):
    aZkmCommand = ZkmCommand(command="retrieve", userName=userName,
                             pageId=pageId, graphicsCaching=0)
    return aZkmCommand

def createRetrieveFromCacheCommand(userName,url):
    aZkmCommand = ZkmCommand(command="retrieveFromCache", userName=userName,
                             url=url, graphicsCaching=0)
    return aZkmCommand

def createQueryCommand(userName, query):
    aZkmCommand = ZkmCommand(command="query", userName=userName, query=query)
    return aZkmCommand

#create command files for zkmServer and zkmBatchDownloader
def createCommandFiles():
    try:
        #get Zope installation folder path
        basePath = os.getcwd()
        zopeString = "zopeHost=localhost zopePort=8080"

        #for Windows
        if os.name != "posix":
            zKnowManPath = os.path.join(basePath, "lib", "python", "Products", "ZKnowMan")
            pythonExePath = os.path.join(basePath, "bin", "python")

            #---------create zkmBatchDownloader command file
            path = zKnowManPath + "\\zkmBatchDownloader.bat"
            file = open(path, "w")

            line1 = pythonExePath + " zkmBatchDownloader.py " + zopeString + "\n"
            line2 = "pause" + "\n"
            file.write(line1 + line2)
            file.close()

            #---------create zkmServer command file
            path = zKnowManPath + "\\zkmServer.bat"
            file = open(path, "w")

            line1 = pythonExePath + " zkmServer.py " + zopeString + "\n"
            line2 = "pause" + "\n"
            file.write(line1 + line2)
            file.close()

        #for Linux
        else:
            zKnowManPath = os.path.join(basePath, "Products", "ZKnowMan")

            #---------create zkmBatchDownloader command file
            path = zKnowManPath + "/zkmBatchDownloader"
            file = open(path, "w")

            line1 = "./zkmBatchDownloader.py " + zopeString + "\n"
            line2 = "read foo" + "\n"
            file.write(line1 + line2)
            file.close()

            os.chmod(path, 0111) #make it executable
            os.chmod(path + ".py", 0111) #make it executable--is this needed?

            #---------create zkmServer command file
            path = zKnowManPath + "/zkmServer"
            file = open(path, "w")

            #---------create zkmServer command file
            line1 = "./zkmServer.py " + zopeString + "\n"
            line2 = "read foo" + "\n"
            file.write(line1 + line2)
            file.close()

            os.chmod(path, 0111) #make it executable
            os.chmod(path + ".py", 0111) #make it executable--is this needed?

    except Exception, data:
        writeError("Error creating zkmServer and zkmBatchDownloader command files: " + `data`)
        raise

    return

#End of ZKnowMan class support functions--------------------------------------------------------------

class ZKnowMan(OFS.Folder.Folder):
    """This is the main ZKnowMan product class."""

    """Zope product infra-structure stuff starts here."""

    name = "ZKnowMan"
    id = "zkm"
    icon = "misc_/ZKnowMan/ZKnowMan"
    meta_type = "ZKnowMan"

    _properties = (
        { "id" : "title", "type" : "string", "mode" : "" },
        { "id" : "tempFolder", "type" : "string", "mode" : "w" },
        { "id" : "mgFolder", "type" : "string", "mode" : "w" },
        { "id" : "htmlFilesFolder", "type" : "string", "mode" : "w" },
        { "id" : "binariesFolder", "type" : "string", "mode" : "w" },
        { "id" : "serverPort", "type" : "int", "mode" : "w" },
        { "id" : "adzapperConfigFileFolder", "type" : "string", "mode" : "w" }
    )

    manage_options=OFS.Folder.Folder.manage_options

    #from Zope 2.2.0 on, you have to explicitly make those methods accessible that can
    #be called from outside of your product, i.e. by Zope using DTML or links (see "Product
    #author's guide to Zope 2.2+ security")
    #__ac_permissions__ = (
    #)

    """Zope product infra-structure stuff ends here."""

    def zkmAddUser(self, REQUEST):
        """Add a ZKnowMan authorized user. Was previously an external method."""
        try:
            acl_users_folder= self["acl_users"]

            name = REQUEST["name"]
            name = string.strip(name)

            password = REQUEST["password"]
            password = string.strip(password)

            confirm = REQUEST["confirm"]
            confirm = string.strip(confirm)

            domains = ""
            roles = ["ZKMUser"]

            if (not name) or (not password) or (not confirm):
                return formatError("Name, password, and confirm fields are required.")

            if confirm != password:
                return formatError("Password and confirmation password did not match. "
                                   + "User could not be added.")

            result = acl_users_folder._addUser(name,password,confirm,roles,domains,REQUEST)
        except:
            writeTracebackAsString(20)
            return formatError("Error adding user. " + RETRY_OR_SEE_LOG)

        return formatMessage("User added successfully.")

    def zkmProxy(self, REQUEST, AUTHENTICATED_USER):
        """Provide a proxy-like service under whose control the user will surf. The proxy-like
        service will cause the surfed pages to be cached for later querying, etc."""
        try:
            status = "determining proxy request"

            aZkmRequest = ZkmRequest(REQUEST, AUTHENTICATED_USER)

            #aZkmRequest.writeTrace()

            status = "opening URL"

            urlFileObj = urllib.urlopen(aZkmRequest.urlToOpen)

            list = urlFileObj.readlines()
            htmlPageString = ""

            for item in list:
                htmlPageString = htmlPageString + item

            status = "adjusting URL if necessary due to redirection"

            aZkmRequest.urlAfterOpen = urlFileObj.geturl()
            urlFileObj.close()
            aZkmRequest.zkmBase = zkmUrl.getUrlBaseFromFullUrl(aZkmRequest.urlAfterOpen)

            status = "determining system configuration information"

            tempFolder = self["tempFolder"]
            serverPort = int(self["serverPort"])

            status = "creating HTML page temp file"

            filePathName = createTempFile(tempFolder, aZkmRequest.userName, htmlPageString)

            status = "converting HTML for return to proxy"

            command = createProxifyCommand(aZkmRequest, filePathName)

            htmlPageString = proxifyPage(command, aZkmRequest.serverName, serverPort)

            status = "saving and indexing HTML page"

            command = createSavePageCommand(aZkmRequest, filePathName)

            saveAndIndexPage(command, aZkmRequest.serverName, serverPort)

            return htmlPageString

        except Exception, data:
            writeError("Error " + status + ": " + `data`)
            writeTracebackAsString(20)

            return formatError("Error " + status + ". " + RETRY_OR_SEE_LOG)

    def zkmOptionsDict(self, REQUEST):
        """Return ZKnowMan options to external helper programs, such as zkmServer."""
        optionsDict = {}
        optionsDict["binariesFolder"] = self["binariesFolder"]
        optionsDict["htmlFilesFolder"] = self["htmlFilesFolder"]
        optionsDict["mgFolder"] = self["mgFolder"]
        optionsDict["tempFolder"] = self["tempFolder"]
        optionsDict["serverPort"] = self["serverPort"]
        optionsDict["adzapperConfigFileFolder"] = self["adzapperConfigFileFolder"]
        return `optionsDict`


    def zkmRetrieve(self, REQUEST, AUTHENTICATED_USER):
        """Retrieve initial cached page from search/query results link that the user clicks on."""
        errorMsg = "Error in 'retrieve' command."
        request = REQUEST
        pageId = request["pageId"]
        httpHost = request["HTTP_HOST"]
        serverName = string.split(httpHost, ":")[0]
        serverPort = int(self["serverPort"])

        userName = `AUTHENTICATED_USER`

        try:
            command = createRetrieveCommand(userName, pageId)
            aZkmSocket = ZkmSocket(command, serverName, serverPort)

            response = aZkmSocket.sendCommandGetResponse()

            if not response[0]:
                writeError(errorMsg, command=command)
            else:
                result = response[1]
                return result

        except Exception, data:
            writeError(errorMsg + `data`)
            writeTrackbackAsString(20)

        result = formatError(errorMsg + RETRY_OR_SEE_LOG)

        return result

    def zkmCache(self, REQUEST, AUTHENTICATED_USER):
        """Retrieve subsequent pages based upon links the user clicks on starting with the
        initially retrieved query results page...as long as the pages are in the cache. When
        you reach the end of the cache, report this to the user."""

        errorMsg = "Error in 'retrieve from cache' command."
        aZkmRequest = ZkmRequest(REQUEST, AUTHENTICATED_USER)
        userName = aZkmRequest.userName
        serverName = aZkmRequest.serverName
        url = aZkmRequest.urlToOpen
        serverPort = int(self["serverPort"])

        try:
            command = createRetrieveFromCacheCommand(userName, url)
            aZkmSocket = ZkmSocket(command, serverName, serverPort)

            response = aZkmSocket.sendCommandGetResponse()

            if not response[0]:
                writeError(errorMsg, command=command)
            else:
                result = response[1]
                return result

        except Exception, data:
            writeError(errorMsg + `data`)
            writeTrackbackAsString(20)

        result = formatError(errorMsg + "<br>" + url + "<br>" + RETRY_OR_SEE_LOG)

        return result

    def zkmSearch(self, REQUEST, AUTHENTICATED_USER):
        """Respond to the user's query by returning a page of links to pages in the cache
        that contain the query words."""
        errorMsg = "Error in 'query' command."
        request = REQUEST
        query = request["query"]
        httpHost = request["HTTP_HOST"]
        serverName = string.split(httpHost, ":")[0]
        serverPort = int(self["serverPort"])

        userName = `AUTHENTICATED_USER`

        try:
            command = createQueryCommand(userName, query)
            aZkmSocket = ZkmSocket(command, serverName, serverPort)

            response = aZkmSocket.sendCommandGetResponse()

            if not response[0]:
                writeError(errorMsg, command=command)
            else:
                result = response[1]
                return result

        except Exception, data:
            writeError(errorMsg + `data`)
            writeTrackbackAsString(20)

        result = formatError(errorMsg + RETRY_OR_SEE_LOG)

        return result

