#!/usr/bin/python

#zkmDb module

import zkmPath

import ZODB
from ZODB.FileStorage import FileStorage

import Persistence
from Persistence import Persistent

from Products.ZCatalog.Catalog import Catalog
from Products.ZCatalog import Vocabulary

class ZkmPageDb:
    def __init__(self, htmlFilesFolder):
        self.file = htmlFilesFolder + "\\pageDb.fs"
        self.db = ZODB.DB(FileStorage(self.file))
        self.co = self.db.open()
        self.root = self.co.root()

        #if we opened an existing ZODB with a catalog,
        #point the ZkmPageDb's cat (catalog) attribute to the ZODB's cat (catalog)
        if self.root.has_key("cat"):
            self.cat = self.root["cat"]

        #otherwise, this is a new ZODB; so, create and init the catalog and set a "pointer" to it
        else:
            #full-text indices need vocabularies
            v = Vocabulary.Vocabulary('Vocabulary', 'Vocabulary', globbing=0)
            self.cat = Catalog(v)
            self.cat.aq_parent = self.root
            self.root['cat'] = self.cat

            #columns are needed for whatever you want to report on
            self.cat.addColumn("url")
            self.cat.addColumn("fields")

            #indices are needed for whatever you want to search on;
            #text index only works on words, meaning, something that starts with an alpha
            #and doesn't contain any special characters; numbers such as 1234 aren't indexed,
            #but a word such as number1234 is indexed.
            self.cat.addIndex("url", "FieldIndex")
            self.cat.addIndex("content", "TextIndex")

            get_transaction().commit()
        return

    def close(self):
        self.db.close()
        return

    def storeZkmPage(self, aZkmPage):
        uid = id(aZkmPage)
        aZkmPage.fields["uid"] = `uid`
        aZkmPage.content = aZkmPage.content + " uid" + `uid` + " "
        self.root[uid] = aZkmPage
        self.cat.catalogObject(aZkmPage, uid)
        get_transaction().commit()
        return

class ZkmPage(Persistent):
    #simple class to wrap information about an HTML file for querying purposes
    def __init__(self, commandDict, content):
        self.url = commandDict["url"]

        self.fields = {}
        self.fields["dateAndTime"] = commandDict["dateAndTime"]
        self.fields["filePathName"] = commandDict["filePathName"]
        self.fields["userName"] = commandDict["userName"]

        self.content = content
        return
