#!/bin/python
#zkmServer stand-alone program

from zkmTrace import showError, writeTrace, writeTracebackAsString
from zkmZope import getOptionsDictFromZope
from zkmCommand import deserialize
from zkmSocket import cleanupSocket
from zkmDb import ZkmPageDb, ZkmPage
import zkmFile
import zkmHtml
import zkmUrl

import Porter

import socket
import string
import select
import time
import sys
import os

#any settings in this dictionary DON'T need to be
#passed to this module as part of a ZkmCommand
optionsDict = {}

#used to keep track of date and time for user's current search session so we
#can pull the correct pages from the DB/cache as the user accesses links
searchSessionDict = {}

def addMetaDataAndSavePageToFile(commandDict):
    filePathName, url, dateAndTime, userName = (commandDict["filePathName"], commandDict["url"],
                                                commandDict["dateAndTime"], commandDict["userName"])
    htmlFilesFolder = optionsDict["htmlFilesFolder"]
    s = zkmFile.readFileIntoString(filePathName)
    s = zkmHtml.addMetaDataToHtmlString(s, url, dateAndTime, userName)
    filePathName = zkmFile.createHtmlFileName(htmlFilesFolder, userName)
    zkmFile.writeStringToFile(filePathName, s)
    return filePathName

def handleSaveCommand(aZkmPageDb, commandDict):
    graphicsCaching = commandDict["graphicsCaching"]
    tempFilePathName = commandDict["filePathName"]
    filePathName = addMetaDataAndSavePageToFile(commandDict)
    s = zkmHtml.removeHtml(filePathName)
    commandDict["filePathName"] = filePathName
    aZkmPage = ZkmPage(commandDict, s)
    aZkmPageDb.storeZkmPage(aZkmPage)
    if graphicsCaching:
        zkmFile.writeStringToFile(tempFilePathName, filePathName)
        baseName = os.path.basename(tempFilePathName)
        dirName = os.path.dirname(tempFilePathName)
        os.rename(tempFilePathName, os.path.join(dirName, "zkmBd_" + baseName))
    else:
        os.remove(tempFilePathName)
    return

def resultsUrlDateAndTimeComparison(one, two):
    oneField = one[1] + one[2]
    twoField = two[1] + two[2]
    return cmp(oneField, twoField)

def handleQueryCommand(aZkmPageDb, commandDict, handlersock):
    query = commandDict["query"]
    #writeDebugInfo("query=" + `query` + "\n")

    #first stem the query since the text has been stemmed before storage in ZODB
    queryList = string.split(query, " ")

    #hadEndingParen = []

    #for i in range(0, len(queryList)):
    #   if string.find(queryList[i], ")") != -1:
    #       hadEndingParen.append(1)
    #   else:
    #       hadEndingParen.append(0)

    queryList = Porter.stem(queryList)

    query = ""
    for i in range(0, len(queryList)):
        #if hadEndingParen[i]:
        #   if string.find(queryList[i], ")") == -1:
        #       query = query + queryList[i] + ")" + " "
        #   else:
        #       query = query + queryList[i] + " "
        #else:
        query = query + queryList[i] + " "

    query = string.strip(query)

    #writeDebugInfo("query after stemming =" + `query` + "\n")

    #now perform the query
    results = aZkmPageDb.cat({"content" : query})

    s = ("<html>" + "\n"
        + "\t<head>" + "\n"
        + "\t</head>" + "\n"
        + "\t<body>" + "\n"
        + "<h1>Search Results</h1>" + "<br>\n\n")

    if results:
        #results are returned as "brains" which can't be sorted, so we must put in a list
        listOfResults = []

        for result in results:
            listOfResults.append((result.fields["uid"], result.url, result.fields["dateAndTime"]))

        listOfResults.sort(resultsUrlDateAndTimeComparison)

        for result in listOfResults:
            s = s + ('\t\t<a href="zkmRetrieve?pageId=' + result[0] + '">'
                    + '<font size="5">' + result[1] + '</font>'
                    + '</a>'
                    + " at "
                    + time.ctime(float(result[2])) + "<br><br>\n")
    else:
        s = s + '<font size="5">No matches found.</font><br><br>\n'

    s = s + ("\t</body>" + "\n"
            + "</html>" + "\n")

    handlersock.send(s)
    return

def recordCacheDateTime(dateAndTime, userName):
    #record date and time of initial search results link taken; it controls the date and time of
    #subsequent pages that are retrieved from the cache as the user follows links from the
    #initial page that is retrieved.

    global searchSessionDict

    #dateAndTime is stored as a float of `time.time()` that was passed to this program
    searchSessionDict[userName] = dateAndTime
    return

def handleRetrieveCommand(aZkmPageDb, commandDict, handlersock,
                          serverName, serverPort, binariesFolder):
    pageId = commandDict["pageId"]

    results = aZkmPageDb.cat({"content" : "uid" + pageId})
    result = results[0]

    filePathName = result.fields["filePathName"]
    userName = result.fields["userName"]
    dateAndTime = float(result.fields["dateAndTime"])
    url = result.url

    parmsDict = {}
    parmsDict["zkmBase"] = zkmUrl.getUrlBaseFromFullUrl(url)
    parmsDict["serverName"] = serverName
    parmsDict["serverPort"] = `serverPort`
    parmsDict["graphicsCaching"] = 0
    parmsDict["binariesFolder"] = binariesFolder

    recordCacheDateTime(dateAndTime, userName)

    htmlPageString = zkmFile.readFileIntoString(filePathName)
    htmlPageString = zkmHtml.directUserToCache(parmsDict, htmlPageString)
    handlersock.send(htmlPageString)
    return

def getBestMatchingZcPage(zcResults, userName, aZkmPageDb):
    if len(zcResults) == 0:
        return None, None

    sessionDateTime = searchSessionDict[userName]
    closestDateTime = float(0)
    bestPageId = None

    for result in zcResults:
        pageId = result.fields["uid"]
        dateAndTime = float(result.fields["dateAndTime"])

        if (sessionDateTime - dateAndTime) < (sessionDateTime - closestDateTime):
            closestDateTime = dateAndTime
            bestPageId = pageId

    if bestPageId == None:
        return None, None
    else:
        results = aZkmPageDb.cat({"content" : "uid" + bestPageId})
        result = results[0]
        filePathName = result.fields["filePathName"]

        zcPage = zkmFile.readFileIntoString(filePathName)
        zcDateTime = closestDateTime

    return zcPage, zcDateTime

def handleRetrieveFromCacheCommand(aZkmPageDb, commandDict, handlersock,
                                   serverName, serverPort, binariesFolder):
    url = commandDict["url"]
    userName = commandDict["userName"]

    parmsDict = {}
    parmsDict["zkmBase"] = zkmUrl.getUrlBaseFromFullUrl(url)
    parmsDict["serverName"] = serverName
    parmsDict["serverPort"] = `serverPort`
    parmsDict["graphicsCaching"] = 0
    parmsDict["binariesFolder"] = binariesFolder

    zcResults = aZkmPageDb.cat({"url" : url})
    zcPage, zcDateTime = getBestMatchingZcPage(zcResults, userName, aZkmPageDb)

    s = zcPage

    if s == None:
        s = ("<html><head></head><body><center><h3>Page for that link is not in the cache: <br>"
            + url
            + "</h3></center></body></html>")
    else:
        s = zkmHtml.directUserToCache(parmsDict, s)

    handlersock.send(s)
    return

def handleProxifyCommand(commandDict, handlersock, serverName, serverPort):
    url = commandDict["url"]
    graphicsCaching = commandDict["graphicsCaching"]
    filePathName = commandDict["filePathName"]

    parmsDict = {}
    parmsDict["zkmBase"] = zkmUrl.getUrlBaseFromFullUrl(url)
    parmsDict["serverName"] = serverName
    parmsDict["serverPort"] = `serverPort`
    parmsDict["graphicsCaching"] = graphicsCaching
    parmsDict["binariesFolder"] = ""

    htmlPageString = zkmFile.readFileIntoString(filePathName)
    htmlPageString = zkmHtml.proxifyHtml(parmsDict, htmlPageString)
    handlersock.send(htmlPageString)
    return

def serve(zopeHost, zopePort):
    global optionsDict
    optionsDict = getOptionsDictFromZope(zopeHost, zopePort)
    serverPort = int(optionsDict["serverPort"])
    htmlFilesFolder = optionsDict["htmlFilesFolder"]
    binariesFolder = optionsDict["binariesFolder"]
    #zkmHtml.makeCombinedEngine(optionsDict["adzapperConfigFileDirectory"])
    aZkmPageDb = ZkmPageDb(htmlFilesFolder)
    HOST = ''
    serversock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    serversock.bind(HOST, serverPort)
    serversock.listen(5)
    print "zkmServer is ready for requests; listening on port " + `serverPort` + "\n"

    try:
        while 1:
            time.sleep(0.1)

            input, output, exc = select.select([serversock], [], [], 0)

            for sock in input:
                try:
                    handlersock, addr = sock.accept()
                except Exception, data:
                    showError("zkmServer - Error accepting client socket: " + `data`)
                    break

                try:
                    rcvdata = handlersock.recv(1024)
                except Exception, data:
                    cleanupSocket(handlersock)
                    showError("zkmServer - Error receiving client data: " + `data`)
                    break

                try:
                    commandDict = deserialize(rcvdata)
                    command = commandDict["command"]

                    if command == "save":
                        handlersock.close()
                        handleSaveCommand(aZkmPageDb, commandDict)

                    elif command == "proxify":
                        handleProxifyCommand(commandDict, handlersock, zopeHost, zopePort)
                        handlersock.close()

                    elif command == "query":
                        handleQueryCommand(aZkmPageDb, commandDict, handlersock)
                        handlersock.close()

                    elif command == "retrieve":
                        handleRetrieveCommand(aZkmPageDb, commandDict, handlersock,
                                              zopeHost, zopePort, binariesFolder)
                        handlersock.close()

                    elif command == "retrieveFromCache":
                        handleRetrieveFromCacheCommand(aZkmPageDb, commandDict, handlersock,
                                                       zopeHost, zopePort, binariesFolder)
                        handlersock.close()
                    else:
                        handlersock.close()

                except Exception, data:
                    cleanupSocket(handlersock)
                    showError("zkmServer - Error processing client request: " + `data`)
                    writeTracebackAsString(20)
                    try:
                        showError(rcvdata)
                    except:
                        pass
                    break

    except Exception, data:
        showError("zkmServer - Error: " + `data`)
        cleanupSocket(serversock)
        try:
            aZkmPageDb.close()
        except:
            showError("zkmServer - Error closing 'pageDb' database: " + `data`)

    return

if __name__ == "__main__":
    try:
        argsError = 0
        arg1, val1 = string.split(sys.argv[1], "=")
        arg2, val2 = string.split(sys.argv[2], "=")
        zopeHost = string.strip(val1)
        zopePort = int(string.strip(val2))
    except:
        argsError = 1

    if argsError or (arg1 != "zopeHost") or (arg2 != "zopePort"):
        print "Usage: python zkmServer.py zopeHost=hostname zopePort=nnnn\n"
        print "Invalid command arguments. Please see usage note above."
        sys.exit(-1)

    serve(zopeHost, zopePort)
