#!/usr/bin/python

#zkmUrl module

import string
import urlparse
import os.path
import re

#compiled regular expressions
decodeUrlPat = re.compile(r'XZQ(?P<number>...)', re.IGNORECASE | re.DOTALL)

def getUrlBaseFromFullUrl(fullUrl):
    #fullUrl passed must be a full url starting with http:// or https://
    urlStart = string.lower(fullUrl[:10])

    assert urlStart[:7] == "http://" or urlStart[:8] == "https://", "getUrlBaseFromFullUrl fullUrl error: " + fullUrl

    urlTuple = urlparse.urlparse(fullUrl)
    urlBase = urlTuple[0] + "://" + urlTuple[1] + urlTuple[2]

    #Chop off end part if it refers to a document versus a folder; keep the ending /
    i = string.rfind(urlBase, "/")
    j = string.rfind(urlBase, ".")

    if i != -1:
        if urlBase[i-1] != "/": #watch out for beginning // in http://, for instance
            if j > i:
                urlBase = urlBase[:i+1]

    return urlBase

def isNonHtmlUrl(url):
    urlStart = string.lower(url[:10])

    if urlStart[:6] == "ftp://":
        return 1
    elif urlStart[:5] == "news:":
        return 1
    elif urlStart[:7] == "mailto:":
        return 1
    elif urlStart[:8] == "file:///":
        return 1
    elif urlStart[:7] == "gopher:":
        return 1
    elif urlStart[:5] == "nntp:":
        return 1
    elif urlStart[:7] == "telnet:":
        return 1
    elif urlStart[:5] == "wais:":
        return 1

    return 0

def constructAbsoluteUrl(urlBase, url):
    #urlBase passed must start with http:// or https://
    #Only attempt to join with base once the url has been verified to not
    #begin with any of the standard schemes.
    urlStart = string.lower(urlBase[:10])

    assert urlStart[:7] == "http://" or urlStart[:8] == "https://", "constructAbsoluteUrl urlBase error: " + urlBase

    urlStart = string.lower(url[:10])

    if urlStart[:7] == "http://" or urlStart[:8] == "https://":
        return url
    elif urlStart[:1] == "#":
        return url
    elif isNonHtmlUrl(urlStart):
        return url
    else:
        return urlparse.urljoin(urlBase, url)

def constructProxyUrl(httpServerName, httpServerPort, urlBase, url, graphicsCaching):
    proxyUrl = ("http://" + httpServerName + ":" + httpServerPort
                + '/zkm/zkmProxy?zkmUrl=' + constructAbsoluteUrl(urlBase,url))
    if graphicsCaching:
        proxyUrl = proxyUrl + "&zkmGc=1"
    return proxyUrl

def getUrlFileExtension(url):
    baseName = os.path.basename(url)
    name, ext = os.path.splitext(baseName)
    if ext:
        ext = string.lower(ext)
    return ext

def isUrlForDisplayableImage(url):
    #return true if url extension is for an image file which is
    #displayable by all the major browsers

    inStandardExtensions = getUrlFileExtension(url) in (".gif", ".jpeg", ".jpe", ".jpg", ".png")

    i = string.rfind(url, "_")
    if i == -1:
        inUnderscoreExtensions = 0
    else:
        s = url[i:]
        inUnderscoreExtensions = s in ("_gif", "_jpeg", "_jpe", "_jpg", "_png")

    return inStandardExtensions or inUnderscoreExtensions

def isUrlForPdfFile(url):
    return getUrlFileExtension(url) in (".pdf",)

def filePathNameFromFullUrl(baseFolder, fullUrl):
    if fullUrl[:7] == "http://":
        fullUrl = fullUrl[7:]
    elif fullUrl[:8] == "https://":
        fullUrl = fullUrl[8:]

    fullUrl = string.replace(fullUrl, ":", "_")

    dirName = os.path.dirname(fullUrl)
    dirName = string.replace(dirName, ".", "/")
    baseName = os.path.basename(fullUrl)
    fullUrl = dirName + "/" + baseName

    filePathName = baseFolder + "/" + fullUrl
    filePathName = string.replace(filePathName, "/", "\\")
    return filePathName

def filePathNameToFileUrl(filePathName):
    #example:
    #from: H:\WebSite\Extensions\zkmFiles\binaries\www\dmoz\org\img\odphead.gif
    #to: file:///H|/WebSite/Extensions/zkmFiles/binaries/www/dmoz/org/img/odphead.gif

    fileUrl = string.replace(filePathName, "\\", "/")
    fileUrl = string.replace(fileUrl, ":", "|")
    fileUrl = "file:///" + fileUrl

    return fileUrl

def fixedStrOrdOfChar(c):
    #convert character to fixed length stringified ORD of a character by
    #prepending it with a zero string if necessary, such as "0" + "12"
    ordStr = `ord(c)`
    if len(ordStr) < 3: ordStr = "0" + ordStr
    return ordStr

def encodeUrlForMg(s):
    #This is used only to encode the URL for MG that we want to be able to find later
    #using MGs full-text-index-based search capability. We are encoding the URL so that
    #when MG does the index, it won't split up the URL.
    #Use the built-in ord(c) function along with special alpha char sequence ZYXQ.
    #So, / would become ZYXQ + ord("/")
    newS = ""
    for c in s:
        if c in string.letters or c in string.digits:
            newS = newS + c
        else:
            newS = newS + "XZQ" + fixedStrOrdOfChar(c)
    return newS

def strOrdToCharReplaceFunction(match):
    strOrdOfChar = match.group("number")
    s = chr(int(strOrdOfChar))
    return s

def decodeUrlFromMg(s):
    #This is used only to decode the URL for MG that we encoded so it wouldn't be split apart
    #when the document containing it was indexed.

    #do chr(int(s)) to get back the original character--the number s
    #will always be three chars long
    s = decodeUrlPat.sub(strOrdToCharReplaceFunction, s)

    return s


